#version 330

in vec3 position;
in vec3 normal;
in vec2 uv;
in vec3 color;
in vec3 tangent;
in vec3 bi_tangent;

out vec2 fragmentUV;
out vec3 P;
out vec3 N;
out vec3 C;
out mat3 localSurface2World;
out vec3 tang_test;
out mat3 normalMatrix;
out vec4 ShadowCoord;

uniform mat4 modelMatrix;
uniform mat4 projectionMatrix;
uniform mat4 viewMatrix;
uniform mat4 depthMVP;


void main(){
	
	fragmentUV = uv;
	mat4 normalMatrix = inverse(transpose(modelMatrix));
	vec3 worldNormal = normalize( (normalMatrix * vec4(normal, 0)).xyz );
	gl_Position = projectionMatrix * viewMatrix * modelMatrix * vec4(position, 1);
	mat4 biasMatrix = mat4(
		0.5, 0.0, 0.0, 0.0,
		0.0, 0.5, 0.0, 0.0,
		0.0, 0.0, 0.5, 0.0,
		0.5, 0.5, 0.5, 1.0
	);
	ShadowCoord = biasMatrix * depthMVP * modelMatrix * vec4(position, 1);
	
	N = normalize(worldNormal);
	P = ( modelMatrix * vec4(position, 1) ).xyz;

	tang_test = normalize(tangent);

	// calculate TBN Matrix
	localSurface2World[0] = normalize((normalMatrix * vec4(tangent, 0.0)).xyz);					// TANGENT
	localSurface2World[2] = normalize(worldNormal);												// NORMAL

	localSurface2World[1] = normalize(cross(localSurface2World[2], localSurface2World[0]));		// BITANGENT = TANGENT x NORMAL

}