#version 330

//input values form shader
in vec3 worldNormal;
in vec2 fragmentUV;
in vec3 position_world;
in vec3 direction_camera;
in vec3 direction_light;

//output values
layout(location = 0) out vec4 fragColor;

//constant values
uniform sampler2D colorTexture;
uniform mat4 modelMatrix;
uniform mat4 viewMatrix;
uniform vec3 position_light;

void main() {

// light color and intensity
vec3 lightColor = vec3(1,1,1);
float lightIntensity = 1.0f;

// material
vec3 diffusePart = texture2D( colorTexture, fragmentUV ).rgb;
vec3 ambientPart = vec3(0.6,0.6,0.6) * diffusePart;
vec3 specularPart = vec3(0.5,0.5,0.5);
//vec3 specularPart = vec3(1,1,1);

// distance to the light
float distance = length(position_light - position_world);

// worldNormal of vertex
vec3 n = normalize( worldNormal );

// direction from light to fragment
vec3 l = normalize( direction_light );

// Cosine of the angle between the normal and the light direction
float cosTheta = clamp( dot( n,l ), 0,1 );
//float cosTheta = max(0,dot( n,l ));

// direction towards camera
vec3 v = normalize(direction_camera);

// reflection-direction
vec3 R = reflect(-l,n);
vec3 h = (v + l) / length(v + l);

// Cosine of the angle between camera- and reflection- direction
float cosAlpha = clamp( dot( v,R ), 0,1 );
//float cosAlpha = max(0,dot( n,h ));

fragColor = vec4(ambientPart + lightIntensity * diffusePart * lightColor * cosTheta / (distance*distance) + lightIntensity * specularPart * lightColor * pow(cosAlpha,50), 1.0);
//fragColor = vec4(ambientPart + lightIntensity * diffusePart * lightColor * cosTheta / (distance*distance) + lightIntensity * specularPart * lightColor * pow(cosAlpha, 5) / (distance*distance), 1.0);


}
