#version 330
//input values from shader
in vec3 position;  //vertex position in local space
in vec3 normal;    //vertex normal in local space
in vec2 uv;        //vertex uv-coords

//output values
out vec3 worldNormal;
out vec2 fragmentUV;
out vec3 position_world;
out vec3 direction_camera;
out vec3 direction_light;

//constant values
uniform mat4 modelMatrix;
uniform mat4 viewMatrix;
uniform mat4 projectionMatrix; 
uniform vec3 position_light;   //light position in world space

void main() {

// uv coords
fragmentUV = uv;
// output position
gl_Position = projectionMatrix * viewMatrix * modelMatrix * vec4(position, 1);

// vertex position after transform in worldspace
position_world = (modelMatrix * vec4(position,1)).xyz;

// direction from vertex towards camera
vec3 position_cameraspace = ( viewMatrix * modelMatrix * vec4(position, 1)).xyz;
direction_camera = vec3(0,0,0) - position_cameraspace;

// direction from vertex towards light
vec3 position_light_cameraspace = ( viewMatrix * vec4(position_light, 1)).xyz;
direction_light = position_light_cameraspace + direction_camera;

// normal of the vertex in worldspace
worldNormal = ( viewMatrix * modelMatrix * vec4(normal, 0)).xyz;
}